﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Routing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;

namespace ConstraintRouting.Tests.Routes {
    
    [TestClass]
    public class RouteTests {
    
        [TestMethod]
        public void Custom_Route_When_Param_Length_Greater_5_Should_Use_Custom_Route()
        {
            MockRepository mocks = new MockRepository();

            var httpContext = mocks.Stub<HttpContextBase>();
            var httpRequest = mocks.Stub<HttpRequestBase>();

            using (mocks.Record()) {

                SetupResult.For(httpRequest.AppRelativeCurrentExecutionFilePath).Return("~/Home/Custom/123456");
                SetupResult.For(httpContext.Request).Return(httpRequest);
            }


            var routes = new RouteCollection();
            MvcApplication.RegisterRoutes(routes);

            using (mocks.Playback()) {
         
                var route = routes.GetRouteData(httpContext);

         
                Assert.AreEqual("Home", route.Values["controller"]);
                Assert.AreEqual("Custom", route.Values["action"]);
                Assert.AreEqual("123456", route.Values["id"]);
            }
        }   
        
        [TestMethod]
        public void Custom_Route_When_Param_Length_Lower_Than_5_Should_Not_Find_Route()
        {
            MockRepository mocks = new MockRepository();

            var httpContext = mocks.Stub<HttpContextBase>();
            var httpRequest = mocks.Stub<HttpRequestBase>();

            using (mocks.Record()) {

                SetupResult.For(httpRequest.AppRelativeCurrentExecutionFilePath).Return("~/Home/Custom/1234");
                SetupResult.For(httpContext.Request).Return(httpRequest);
            }


            var routes = new RouteCollection();
            MvcApplication.RegisterRoutes(routes);

            using (mocks.Playback()) {
         
                var route = routes.GetRouteData(httpContext);

                Assert.IsNull(route);
            }
        }
    
    }
    
        
}
