﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Transactions;

namespace Model {

    public class CustomerDAO : ICustomerDAO
    {
    
        public Customer Save(Customer customer)
        {
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.ConnectionStrings["db"].ConnectionString))
            {
                using (TransactionScope transactionScope = new TransactionScope())
                {
                    SqlCommand command = new SqlCommand();
                    command.Connection = connection;

                    if (customer.Id == null) {
                        command.CommandText =
                            "insert into Customer (NameFirst,NameLast,Email,DateOfBirth,IsActive,Title) values (@NameFirst,@NameLast,@Email,@DateOfBirth,@IsActive,@Title);select Scope_Identity();";

                    } else {
                        command.CommandText =
                            "update Customer set NameFirst=@NameFirst,NameLast=@NameLast,Email=@Email,DateOfBirth=@DateOfBirth,IsActive=@IsActive,Title=@Title where CustomerId=@CustomerId";
                        command.Parameters.AddWithValue("@CustomerId", customer.Id);
                    }
                    command.Parameters.AddWithValue("NameFirst", customer.Name.NameFirst);
                    command.Parameters.AddWithValue("NameLast", customer.Name.NameLast);
                    command.Parameters.AddWithValue("Email", customer.Email);
                    command.Parameters.AddWithValue("DateOfBirth", customer.DateOfBirth);
                    command.Parameters.AddWithValue("IsActive", customer.IsActive);
                    command.Parameters.AddWithValue("Title", customer.Title);

                    connection.Open();
                    int? currentId = customer.Id;
                    if (currentId == null)
                    {
                        object obj = command.ExecuteScalar();
                        if (obj != null) {
                            currentId = Convert.ToInt32(obj);
                            customer.Id = currentId;
                        }
                    } else
                    {
                        command.ExecuteNonQuery();
                    }

                    if (currentId == null) {
                        throw new Exception("Something went wrong saving a customer...");
                    }

                    command.Parameters.Clear();
                    
                    command.CommandText = "delete from CustomerAddress where CustomerId=@CustomerId";
                    command.Parameters.AddWithValue("CustomerId", currentId);

                    command.ExecuteNonQuery();
                    command.CommandText = "insert into CustomerAddress values (@Street,@City,@State,@Country,@CustomerId)";
                    command.Parameters.Add("Street", SqlDbType.VarChar);
                    command.Parameters.Add("City", SqlDbType.VarChar);
                    command.Parameters.Add("State", SqlDbType.VarChar);
                    command.Parameters.Add("Country", SqlDbType.VarChar);
                    
                    foreach (CustomerAddress address in customer.Addresses) {
                    
                        command.Parameters["Street"].Value = address.Street;
                        command.Parameters["City"].Value = address.City;
                        command.Parameters["State"].Value = address.State;
                        command.Parameters["Country"].Value = address.Country;

                        command.ExecuteNonQuery();
                    }
                    
                    transactionScope.Complete();
                }
                                
            }
            return customer;
        }
        
        
        public Customer GetCustomerById(int id)
        {
            IList<Customer> customers = GetCustomers(SearchCriteria.Id, id, ListOrder.Asc);
            if (customers == null)
            {
                throw new Exception("Customer not found");
            }
            return customers[0];
        }

        public IList<Customer> GetCustomers(ListOrder order)
        {
            return GetCustomers(SearchCriteria.None, null, order);
        }

        public IList<Customer> GetCustomers()
        {
            return GetCustomers(SearchCriteria.None, null, ListOrder.Asc);
        }

        public IList<Customer> GetCustomersByName(string name)
        {
            return GetCustomers(SearchCriteria.Name, name, ListOrder.Asc);
        }

        public IList<FlatCustomer> GetFlatCustomers()
        {
            List<FlatCustomer> customers = new List<FlatCustomer>();

            using (SqlConnection connection = new SqlConnection(ConfigurationManager.ConnectionStrings["db"].ConnectionString))
            {
                SqlCommand command = new SqlCommand();
                command.Connection = connection;

                command.CommandText =
                     "select CustomerId,NameFirst,NameLast,Email,Title from Customer";
                connection.Open();

                SqlDataReader reader = command.ExecuteReader();
                while (reader.Read())
                {
                    FlatCustomer customer = new FlatCustomer()
                    {
                        Id = reader.GetInt32(0),
                        NameFirst = reader.GetString(1),
                        NameLast = reader.GetString(2),
                        Email = reader.GetString(3),
                        Title = reader.GetString(4)
                    };

                    customers.Add(customer);
                }

            }

            return customers;  
        }

        public int GetCustomerCount()
        {
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.ConnectionStrings["db"].ConnectionString))
            {
                SqlCommand command = new SqlCommand("select count(CustomerId) from Customer", connection);

                connection.Open();
                object obj = command.ExecuteScalar();
                if (obj !=null)
                {
                    int i = (int) obj;
                    return i;
                } else
                {
                    return 0; // This is just so so bad....
                }
            }
        }

        private IList<Customer> GetCustomers(SearchCriteria criteria, object value, ListOrder order) 
        {
            IList<Customer> customers = new List<Customer>();
            
            using (SqlConnection connection = new SqlConnection(ConfigurationManager.ConnectionStrings["db"].ConnectionString))
            {
                SqlCommand command = new SqlCommand();
                command.Connection = connection;

                string sortOrder = "";
                
                switch (order)
                {
                    case ListOrder.Asc:
                        sortOrder = " order by CustomerId asc";
                        break;
                    case ListOrder.Desc:
                        sortOrder = " order by CustomerId desc";
                        break;
                }
                
                switch (criteria)
                {
                    case SearchCriteria.None:
                        command.CommandText =
                            "select CustomerId,NameFirst,NameLast,Email,DateOfBirth,IsActive,Title from Customer";
                        break;
                    case SearchCriteria.Id:
                        command.CommandText =
                            "select CustomerId,NameFirst,NameLast,Email,DateOfBirth,IsActive,Title from Customer where CustomerId=@CustomerId";
                        command.Parameters.AddWithValue("CustomerId", (int)value);
                        break;
                    case SearchCriteria.Name:
                        command.CommandText =
                            "select CustomerId,NameFirst,NameLast,Email,DateOfBirth,IsActive,Title from Customer where NameFirst like @NameFirst";
                        command.Parameters.AddWithValue("NameFirst", value + "%");
                        break;
                    default:
                        throw new ArgumentOutOfRangeException("criteria");
                }
                command.CommandText = command.CommandText + sortOrder;
                
                SqlCommand subCommand = new SqlCommand("select Street,City,State,Country from CustomerAddress where CustomerId=@CustomerId", connection);
                subCommand.Parameters.Add("CustomerId", SqlDbType.Int);
                
                connection.Open();
                
                SqlDataReader reader = command.ExecuteReader();
                
                while (reader.Read())
                {
                    Customer customer = new Customer()
                                        {
                                            Id = reader.GetInt32(0),
                                            Name = new CustomerName()
                                                   {
                                                        NameFirst = reader.GetString(1),
                                                        NameLast = reader.GetString(2)
                                                   },
                                            Email = reader.GetString(3),
                                            DateOfBirth = reader.GetDateTime(4),
                                            IsActive = reader.GetBoolean(5),
                                            Title = reader.GetString(6),
                                            Addresses = new List<CustomerAddress>()
                                        };


                    subCommand.Parameters[0].Value = customer.Id;

                    SqlDataReader subReader = subCommand.ExecuteReader();
                    while (subReader.Read())
                    {
                        CustomerAddress address = new CustomerAddress()
                                                  {
                                                      Street = subReader.GetString(0),
                                                      City = subReader.GetString(1),
                                                      State = subReader.GetString(2),
                                                      Country = subReader.GetString(3)
                                                  };
                        customer.Addresses.Add(address);                                                                          
                    }
                    
                    subReader.Close();

                    customers.Add(customer);                                   
                }
                
            }

            return customers;
            
        }
    }

    internal enum SearchCriteria
    {
        None,
        Id,
        Name
    }
}
